import java.awt.*;

/**
*	This class provides an object for holding a single 'case' or
*	'plot' of data.  Each case will consist of three variables: <BR>
*	H<SUB>0</SUB> = Hubble Constant <BR>
*	Omega<SUB>M</SUB> = Mass Factor <BR>
*	Omega<SUB>L</SUB> = Acceleration Factor <BR>
*
*	@author Rob Falck - Case Western Reserve University
**/

public class Case {

	/** Variable for the Hubble Constant **/
	public Variable h0;
	/** Variable for the Mass Factor **/
	public Variable omegaM;
	/** Variable for the Acceleration Factor **/
	public Variable omegaL;
	/** An input field for labelling the case **/
	public TextField inputField;

	/** The color in which this case will be plotted **/
	public Color color = Color.black;

	/** Arrays in which the results of the size integration will be stored.
	*	The calculation takes to long to do dynamically.
	**/
	public double[] tplot = new double[1000], aplot = new double[1000];

	/** The index of tplot which is shown while tracing **/
	public int traceIndex = 0;

	/** Creates a new case: all values initialized to 0.0 **/
	public Case() {
		h0 = new Variable("Hubble Constant",0);
		omegaM = new Variable("Omega Matter",0);
		omegaL = new Variable("Omega Lambda",0);
		inputField = new TextField();

		h0.setMaxValue(600);
		h0.setMinValue(0);
		omegaM.setMaxValue(2.5);
		omegaM.setMinValue(0);
		omegaL.setMaxValue(2.5);
		omegaL.setMinValue(0);
	}

	/** Creates a new case with the specified label **/
	public Case(String label) {
		h0 = new Variable("Hubble Constant",0);
		omegaM = new Variable("Omega Matter",0);
		omegaL = new Variable("Omega Lambda",0);
		inputField = new TextField();
		this.inputField.setText(label);
		h0.setMaxValue(600);
		h0.setMinValue(0);
		omegaM.setMaxValue(2.5);
		omegaM.setMinValue(0);
		omegaL.setMaxValue(2.5);
		omegaL.setMinValue(0);
	}

	/** Returns the label for this case **/
	public String getLabel() {
		return(inputField.getText());
	}

	/** Return the value of H0 - the Hubble Constant **/
	public double getH0() {
		return(h0.value);
	}

	/** Return the value of om - the Mass Parameter **/
	public double getOmegaM() {
		return(omegaM.value);
	}

	/** Return the value of ol - the Acceleration Parameter **/
	public double getOmegaL() {
		return(omegaL.value);
	}

	/**	Set the color in which the plot for this case will be drawn **/
	public void setColor(Color c) {
		this.color = c;
		inputField.setForeground(c);
	}




}
