import java.awt.*;
import java.awt.event.*;


/**
*	The initializing class of the Cosmology applet
*
*	@author Rob Falck - Case Western Reserve University
*	@version 1.0: 05.20.2000-06.08.2000
**/

public class Cosmo extends java.applet.Applet {
	/** The graphing pane on which the output will be plotted **/
	protected GraphPane graph = new GraphPane();

	private CardLayout cl = new CardLayout();

	private Panel inputPane = new Panel(cl);

	private Panel casePane = new Panel();

	private Panel dataPointPane = new Panel(new GridLayout(dataPoints.length+1,4));

	/** When true Cosmo will recalculate the scale of a window before redrawing the plots.
	*	The integration process can take some time on slower machines
	*	and therefore it would not be feasible to recalculate the
	*	size of the window everytime it is redrawn, such as while tracing
	**/
	public static boolean dataIsNew = true;

	/** An array for the cases to be plotted **/
	public static Case[] cases;
	/** An array for the data points to be plotted **/
	public static DataPoint[] dataPoints = new DataPoint[] {
		new DataPoint(0,0,0),
		new DataPoint(0,0,0),
		new DataPoint(0,0,0),
		new DataPoint(0,0,0),
		new DataPoint(0,0,0),
	};


	/**	An array of colors in which the cases will be plotted **/
	public static Color[] colors;


	/** Combobox to choose plot type **/
	public static Choice plotTypeCombo = new Choice();
	///**	Button to toggle the plot type between lookback time and age **/
	//public static Button plotTypeButton = new Button("Display Age");
	/**	Button to switch the displayed panel between cosmology and data point inputs **/
	public static Button editButton = new Button("Edit Data Points");

	/**	Checkbox used to toggle tracing on and off **/
	public static Checkbox traceCheckbox = new Checkbox("Trace");

	/**	The label for the horizontal axis **/
	public static Label xLabel = new Label("Redshift (z)");
	/**	The label for the vertical axis **/
	public static Label yLabel = new Label("tL");
	/** A dummy label to display the units on the horizontal axis below the actual label **/
	private static Label yUnitsLabel = new Label("(GYr)");

	/** The label to display the coordinates of the trace **/
	public static Label traceLabel = new Label();

	/** 0 for Lookback time, 1 for age **/
	public static int plotType=0;

	/** Destroy Cosmo and its cases **/
	public void stop() {
		for(int i=0; i< cases.length; i++) {
			cases[i] = new Case();
		}
		destroy();
	}
	/** Initializes this applet, does the layout **/
	public void init() {
		removeAll();
		graph = new GraphPane();
		cl = new CardLayout();
		inputPane = new Panel(cl);
		casePane = new Panel();
		dataPointPane = new Panel(new GridLayout(dataPoints.length+1,4));

		/** An array for the cases to be plotted **/
		cases = new Case[] {
			new Case("Case 1"),
			new Case("Case 2"),
			new Case("Case 3"),
			new Case("Case 4"),
			new Case("Case 5"),
		};

		/** An array for the data points to be plotted **/
		dataPoints = new DataPoint[] {
			new DataPoint(0,0,0),
			new DataPoint(0,0,0),
			new DataPoint(0,0,0),
			new DataPoint(0,0,0),
			new DataPoint(0,0,0),
		};


		/**	An array of colors in which the cases will be plotted **/
		colors = new Color[] {
				Color.red,
				Color.blue,
				new Color(0,150,0),
				new Color(200,0,200),
				new Color(20,36,75),
			};

		plotTypeCombo = new Choice();
		plotTypeCombo.addItem("Plot Lookback");
		plotTypeCombo.addItem("Plot Age");
		plotTypeCombo.addItem("Plot Size");

		editButton = new Button("Edit Data Points");

		traceCheckbox = new Checkbox("Trace");

		xLabel = new Label("Redshift (z)");
		yLabel = new Label("tL");
		yUnitsLabel = new Label("(GYr)");

		traceLabel = new Label();

		/** 0 for Lookback time, 1 for age **/
		plotType=0;



		setLayout(null);
		add(graph);
		graph.setBounds(50,20,450,250);
		graph.setXMax(6);
		graph.setXMin(0);
		graph.setYMax(10);
		graph.setYMin(0);
		this.setBackground(Color.white);

		add(xLabel);
		xLabel.setBounds(245,270,120,15);
		add(yLabel);
		yLabel.setBounds(13,125,30,15);
		add(yUnitsLabel);
		yUnitsLabel.setBounds(10,140,40,15);

		add(plotTypeCombo);
		plotTypeCombo.setBounds(150,300,125,25);

		add(editButton);
		editButton.setBounds(275,300,125,25);

		add(traceCheckbox);
		traceCheckbox.setBounds(50,275,60,15);
		traceCheckbox.setForeground(Color.gray);

		add(traceLabel);
		traceLabel.setBounds(110,275,100,15);

		inputPane.add(casePane, "Cosmology");
		casePane.setLayout(new GridLayout(cases.length+1,4));
		//add the cases to the casePane
		casePane.add(new Label("Cosmology",Label.CENTER));
		casePane.add(cases[0].h0.label);
		casePane.add(cases[0].omegaM.label);
		casePane.add(cases[0].omegaL.label);
		for(int i=0; i<cases.length; i++) {
			cases[i].setColor(colors[i]);
			casePane.add(cases[i].inputField);
			casePane.add(cases[i].h0.inputField);
			casePane.add(cases[i].omegaM.inputField);
			casePane.add(cases[i].omegaL.inputField);
		}
		inputPane.add(dataPointPane, "Data Points");
		//dataPointPane.setLayout(new GridLayout(dataPoints.length+1,4));
		dataPointPane.add(new Label("Data Point",Label.CENTER));
		dataPointPane.add(dataPoints[0].xValue.label);
		dataPointPane.add(dataPoints[0].yValue.label);
		dataPointPane.add(dataPoints[0].uncertainty.label);
		for(int i=0; i<dataPoints.length; i++) {
			dataPointPane.add(new TextField("Data Point" + (i+1)));
			dataPointPane.add(dataPoints[i].xValue.inputField);
			dataPointPane.add(dataPoints[i].yValue.inputField);
			dataPointPane.add(dataPoints[i].uncertainty.inputField);
		}


		add(inputPane);
		inputPane.setBounds(50,325,450,150);

		plotTypeCombo.addItemListener(new ItemAdapter() {
			public void itemStateChanged(ItemEvent e) {
				plotType = plotTypeCombo.getSelectedIndex();
				if(plotType ==0) {
					yLabel.setText("tL");
					yUnitsLabel.setText("(GYr)");
					xLabel.setText("Redshift (z)");
				} else if(plotType==1) {
					yLabel.setText("Age");
					yUnitsLabel.setText("(GYr)");
					xLabel.setText("Redshift (z)");
				} else if(plotType==2) {
					yLabel.setText("Size");
					yUnitsLabel.setText("(r/r0)");
					xLabel.setText("Time (GYr)");
				}
				dataIsNew = true;
				graph.updateTraceY();
				graph.repaint();
			}
		});


		editButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cl.next(inputPane);
				if(editButton.getLabel().equals("Edit Data Points")) {
					editButton.setLabel("Edit Cosmology");
				} else {
					editButton.setLabel("Edit Data Points");
				}
			}
		});

		traceCheckbox.addItemListener(new ItemAdapter() {
			public void itemStateChanged(ItemEvent e) {
				if(Cosmo.traceCheckbox.getState() == true) {
					graph.updateTraceY();
					Cosmo.traceLabel.setText(FMath.roundOff(graph.traceX,3) + "," + FMath.roundOff(graph.traceY,3));
					graph.requestFocus();
				} else {
					Cosmo.traceLabel.setText("");
				}
				graph.repaint();

			}
		});
	}


	/** Repaint Cosmo's main pane
	*	Currently nothing particular is being done here
	**/
	public void paint(Graphics g) {
		/*g.drawRect(0,0,this.getWidth()-1,this.getHeight()-1);
		g.setColor(Color.black);

		g.drawString("RedShift (z)",245,283);


		}*/
    }


	/**	Computes a simple case (ol=0) for a given cosmology case and redshift z.
	*	Defined in <i>An Introduction to Modern Astrophysics</i> by
	*	Bradley W. Carroll and Dale A. Ostlie
	*
	*	@deprecated Use z2t instead
	**/
	public static double compute(Case c, double z) {
		double result=0;
		double H = c.getH0();
		double Om = c.getOmegaM();
		double Ol = c.getOmegaL();

		// for k>0 and therefore omegaM > 1
		if(Om > 1) {
			result = ((Om/(2*Math.pow((Om-1),1.5)))*(Math.acos((Om*z - Om + 2)/(Om*z + Om))-(2*Math.sqrt((Om-1)*(Om*z+1)))/(Om*(1+z))));
		}
		// for k<0 and therefore omegaM < 1
		else if(Om < 1) {
			result = ((Om/(2*Math.pow((1-Om),1.5)))*(-1*FMath.acosh((Om*z - Om + 2)/(Om*z + Om))+(2*Math.sqrt((1-Om)*(Om*z+1)))/(Om*(1+z))));
		}
		// for k=0 and therefore omegaM = 1
		else {
			result = ( (2./3) * Math.pow((1+z),-1.5));
		}
		return(result);

	}


	/**	Replaces compute to calculate cosmologies of both trivial (Omega<SUB>L</SUB>=0) and nontrivial cases.
	*	Converted from <A HREF="http://www.nhn.ou.edu/~thomas/z2t.html">fortran90 code</A> provied by Rollin C. Thomas of the University of Oklahoma.
	**/
	public static double z2t(Case c, double z) {
		double z2t=0, om, ol;
		double oo=0, b=0;

		om = c.getOmegaM();
		ol = c.getOmegaL();

		double EPS=1.0e-8;

		if((ol > -EPS)&&(ol < EPS)&&(om >= EPS)) {
			// CASE OLZ ol=0.0e0
			z2t=TOLZ(om,z);
		} else if((om > -EPS)&&(om < EPS)&&(ol >= EPS)&&(ol <= 1.0e0-EPS)) {
			// case omz om=0.0e0
			z2t=TOMZ(ol,z);
		} else {
			oo=om+ol;
			if((1.0e0-oo < -EPS)||((1.0e0-oo) > EPS)) {
				b = -13.5e0*om*om*ol/Math.pow((1.0e0-oo),3);
				if((b < EPS)||(b > 2.0e0+EPS)) {
					// CASE A1 b <= 0.0e0 or b>2.0e0
					z2t=TA1(om,ol,z);
				} else {
					// CASE A2 0.0e0 < b <= 2.0e0
					z2t = TA2(om,ol,z);
				}
			} else {
				// Case B om+ol == 1.0
				z2t=TB(om,z);
			}
		}
		//System.out.println("result = " + z2t);
		return(z2t);
	}


	/**
	*	z2t for ol=0.0e0 universe
	*	this function is included for completeness, for ol = 0.0e0
	*	it containts no calls to any elliptic functions
	**/
	public static double TOLZ(double om, double z) {
		double tolz;
		double pre0, pre1, arg0, arg1, arg2, anum, aden, arg3;
		double EPS=1.0e-8;
		// Equation 2 is used and has three forms here
		if( om < 1.0e0-EPS) {
			pre0=(1.0e0-om);
			pre1=(1.0e0/pre0);
			arg0=Math.sqrt(1.0e0+om*z);
			arg1=arg0/(1.0e0+z);
			arg2=0.5e0*om*Math.sqrt(pre1);
			anum=arg0-Math.sqrt(pre0);
			aden=arg0+Math.sqrt(pre0);
			arg3=Math.log(anum/aden);
			tolz=pre1*(arg1+arg2*arg3);
		} else if(om > 1.0e0+EPS) {
			pre0=(1.0e0-om);
			pre1=1.0e0/pre0;
			anum=-pre0;
			aden=1.0e0+om*z;
			arg1=Math.sqrt(aden)/(1.0e0+z);
			arg2=om/Math.sqrt(anum);
			arg3=-Math.atan(Math.sqrt(anum/aden));
			tolz=pre1*(arg1+arg2*arg3);
		} else {
			tolz = 2.0e0*Math.pow((1.0e0+z),(-1.5e0))/3.0e0;
		}
		return(tolz);
	}


	/**	Z2t for 0m=0.0e0 Universe
	*	This function is included for completeness, for om=0.0.
	*	It containts no calls to any elliptic functions, and is good
	*	only for 0.0e0 < ol < 1.0e0.
	**/
	public static double TOMZ(double ol, double z) {
		double tomz;
		double arg0, arg1;
		// Equation 3 is used, only good for 0.0e0 < 0l < 1.0e0.
		arg0=1.0e0/((1.0e0+z)*Math.sqrt((1.0e0/ol)-1.0e0));
		arg1=Math.log(arg0+Math.sqrt(1.0e0+Math.pow(arg0,2)));
		tomz = arg1/Math.sqrt(ol);
		return(tomz);
	}

	/**	Used by zt2 for some non-trivial calculations **/
	public static double TA1(double om, double ol, double z) {
		double TA1,oo,ooo,b,kap,vk,y1,A,k,phinum,phiden,phi,hp,hm,ahatsq;
	    double pre,test0,test1,arg0,arg1,arg2,arg3;
		final double TH=1.0/3.0,EPS=1.0d-8,S2=1.41421356237;
		oo=om+ol;
	    ooo=1.0-oo;
	    b=-13.5*om*om*ol/Math.pow(ooo,3);
	    kap=-ooo/Math.abs(-ooo);
	    vk = Math.pow((kap*(b-1.0)+Math.sqrt(b*(b-2.0))),TH);
	    y1=TH*(kap*(vk+1.0/vk)-1.0);
	    A=Math.sqrt(y1*(3.0*y1+2.0));
	    k=Math.sqrt((2.0*A+kap*(1.0+3.0*y1))/(4.0*A));
	    arg0=kap*y1+om*(1.0+z)/Math.abs(ooo);
	    phinum=arg0-A;
	    phiden=arg0+A;
	    phi=Math.acos(phinum/phiden);
		ahatsq = 0.25*Math.pow((A+kap*y1),2)/(A*kap*y1);
		test0=1.0-ahatsq*Math.sin(phi)*Math.sin(phi);

		if((test0 > -EPS) && (test0 < EPS)) {
			ahatsq=y1*(1.0+y1)/Math.pow((A-kap*y1),2);
			test1=1.0-ahatsq*Math.sin(phi)*Math.sin(phi);
			if((test1>-EPS)&&(test1<EPS)) {
				// EQUATION 24 is used
				arg1=(1.0+z)*om/ooo;
			    phinum=-1.0-arg1;
			    phiden=S2+1.0-arg1;
	         	phi=Math.acos(phinum/phiden);
	         	arg2=(1.0-arg1)*Math.sqrt(Math.pow(arg1,2)+(arg1+1.0/S2)*(1.0+1.0/S2));
	         	arg3=(S2-1.0)*(arg1+S2+1.0)*Math.sqrt((1.0+1.0/S2)*(1.0/S2-arg1));
	         	hm=arg2-arg3;
				hp=arg2+arg3;
				pre=0.25/Math.sqrt(ol);
				arg1=(S2-1.0)*FMath.ellf(phi,Math.sqrt(0.25*(1.0+2.0*S2)));
				arg2=Math.log(hp/hm);
	         	TA1=pre*(arg1+arg2);
			} else {
				// EQUATION 8 is used
				arg1=(1.0+z)*om/ooo;
				arg2=arg1*arg1*(1.0+arg1);
				arg2=Math.sqrt((1.0+y1)*((1.0+y1)*Math.pow(y1,2)-arg2));
				arg2=2.0*kap*y1*arg2;
				arg1=arg1*arg1*(A-kap*y1)-2.0*kap*(1.0+y1)*Math.pow(y1,2);
				hm=arg1-arg2;
				hp=arg1+arg2;
				arg1=0.5*FMath.ellf(phi,k)/(kap*y1*Math.sqrt(A));
				arg2=0.5*(A-kap)*FMath.ellpi(phi,-ahatsq,k)/(y1*(1.0+y1)*Math.sqrt(A));
				arg3=0.5*Math.log(Math.abs(hm/hp))/(kap*y1*Math.sqrt(kap*(y1+1.0)));
				pre=om/Math.pow(Math.abs(ooo),1.5);
	         	TA1=pre*(arg1+arg2+arg3);
			}
		} else {
			// EQUATION 10 is used
			arg1=(1.0+z)*om/ooo;
			hm=(1.0+y1)*(y1-arg1);
			hm=Math.sqrt(hm/(Math.pow(y1,2)+(1.0+arg1)*(y1+arg1)));
			pre=om/Math.sqrt(A*Math.pow(Math.abs(ooo),3));
			arg1=-FMath.ellf(phi,k)/(A+kap*y1);
			arg2=-0.5*(A-kap*y1)*FMath.ellpi(phi,-ahatsq,k)/(kap*y1*(A+kap*y1));

			arg3=-0.5*(Math.sqrt(A/(kap*(y1+1.0)))/(kap*y1));
			arg3=arg3*Math.log(Math.abs((1.0-hm)/(1.0+hm)));
			TA1=pre*(arg1+arg2+arg3);
		}
		return(TA1);
	}

	/**	Used by zt2 for some non-trivial calculations **/
	public static double TA2(double om,double ol,double z) {
		// z2t for om+ol != 1.0e0 Universe, O<b<=2;
		double ta2;
		double oo,ooo,b,arg,arg1,arg2,arg3,arg4,y1=0,y2=0,y3,phi,k,ahatsq;
		final double TH=1.0e0/3.0e0, EPS=1.0e-8, S3=1.73205080757e0;
		oo=om+ol;
		ooo=1.0e0-oo;
		b=-13.5e0*om*om*ol/Math.pow(ooo,3);
		if(   (b>(2.0e0-EPS)) && (b<(2.0e0+EPS))     )   {
			// Equation 19 is used
			arg=(1.0e0+z)*om/ooo;
			y1=Math.sqrt(TH-arg)+1.0e0;
			y1=Math.pow((y1/(y1-2.0e0)),Math.sqrt(1.0e0/TH));
			y2=Math.sqrt(1.0e0-3.0e0*arg)-1.0e0;
			y2=y2/(y2+2.0e0);
			ta2=Math.log(y1*y2)/Math.sqrt(ol);
			return(ta2);
		} else {
			// Equation 15 is used
			arg=TH*Math.acos(1.0e0-b);
			//System.out.println("a1 " + arg);
			arg1=TH*Math.cos(arg);
			//System.out.println("a2 " + arg1);
			arg2=Math.sin(arg)/S3;
			//System.out.println("a3 " + arg2);
			y1=-TH+arg1+arg2;
			y2=-TH-2.0e0*arg1;
			y1=-TH+arg1+arg2;
			y2=-TH-2.0e0*arg1;
			y3=y1-2.0e0*arg2;
			arg3=y1-y2;
			//System.out.println("ooo" + ooo);
			//System.out.println("asin " + Math.sqrt(arg3/(y1-om*(1.0e0+z)/ooo)));
			phi=Math.asin(Math.sqrt(arg3/(y1-om*(1.0e0+z)/ooo)));
			//System.out.println(phi);
			k=Math.sqrt((y1-y3)/arg3);
			ahatsq=y1/arg3;
			arg4=y1*Math.sqrt(arg3*Math.pow(-ooo,3));
			ta2 = 2.0e0*om*(FMath.ellpi(phi,-ahatsq,k)-FMath.ellf(phi,k))/arg4;
			//System.out.println("ELSE" + ta2);
			return(ta2);
		}
	}

	/**	z2t for om+ol==1.0e0 universe **/
	public static double TB(double om, double z) {
		// z2t for om+ol==1.0e0 universe
		double tb, omo,arg;
		// Equation 18 is used
		omo=1.0e0-om;
		arg=Math.sqrt(omo/(om*(1.0e0+z*(3.0e0+z*(3.0e0+z)))));
		tb = 2.0e0*Math.log(arg+Math.sqrt(1.0e0+Math.pow(arg,2)))/(3.0e0*Math.sqrt(omo));
		return(tb);
	}
}
