/**
*	DataPoint serves as a class for data points in the Cosmo applet.
*	They are defined by the user within the applet and overlayed
*	on the Age/Redshift plot.
*
*	@author Rob Falck - Case Western Reserve University
**/
public class DataPoint {

	/**
	*	The value of this point along the x-axis
	*	For the purposes of Cosmo this is a redshift value.
	**/
	Variable xValue = new Variable("Redshift");
	/**
	*	The value of this point along the y-axis
	*	For the purposes of Cosmo this is an age in GYr.
	**/
	Variable yValue = new Variable("Age (GYr)");
	/**
	*	The uncertainty in the y-value of this datapoint.
	*	For the purposes of Cosmo this is in GYr.
	**/
	Variable uncertainty = new Variable("Uncertainty (GYr)");

	/**
	*	Construct a new datapoint at (x,y) with an uncertainty of u.
	**/
	public DataPoint(double x,double y,double u) {
		this.xValue.setValue(x);
		this.yValue.setValue(y);
		this.uncertainty.setValue(u);
	}

	/**
	*	Retrieve the x-value of this datapoint
	**/
	public double getXValue() {
		return(xValue.value);
	}

	/**
	*	Retrieve the y-value of this datapoint
	**/
	public double getYValue() {
			return(yValue.value);
	}

	/**
	*	Retrieve the uncertainty of this datapoint
	**/
	public double getUncertainty() {
			return(uncertainty.value);
	}
}