import java.util.*;

/**
*	FMath is a abstract class with various functions
*	I felt we're either missing or not implemented
*	as I wanted them to be in the JDK Math class
*
*	@author Rob Falck - Case Western Reserve University
*
*	@version 1.01 - 06.08.2000
*
*/
public abstract class FMath {

	/**
	*	Find the index of the maximum value in the inputArray
	*
	*	@return The index of the largest double in the array
	*/
	public static int arrayMax(double[] inputArray) {
		double maximum= inputArray[0];
		int location = 0;
		for(int i=1; i< inputArray.length; i++) {
			maximum = Math.max(maximum,inputArray[i]);
			if(maximum == inputArray[i]) {
				location = i;
			}
		}
		return(location);
	}



	/**
	*	Returns the index of the mimimum value in an array of doubles
	*/
	public static int arrayMin(double[] inputArray) {
		double minimum= inputArray[0];
		int location = 0;
		for(int i=1; i< inputArray.length; i++) {
			minimum = Math.min(minimum,inputArray[i]);
			if(minimum == inputArray[i]) {
				location = i;
			}
		}
		return(location);
	}

	/**
	*	Performs least squares of order ORDER on independent x and
	*	dependent y
	**/
	public static double[] leastSquares(double[] x, double[] y, int ORDER) {
		/* Format for all matrices is matrix[row][column] */
		try {
			double[][] normalArray = new double[ORDER+1][ORDER+2];
			for(int i=0; i<=ORDER; i++) {
				for(int j=0; j< y.length; j++) {
					normalArray[i][ORDER+1] += (Math.pow(x[j],i))*y[j];
				}
				for(int j=0; j<=ORDER; j++) {
					for(int k=0; k<y.length; k++) {
						normalArray[i][j] += Math.pow(x[k],(j+i));
					}
				}
			}return(gaussElim(normalArray));
		} catch(Exception e) {
			double[] returnArray = new double[0];
			return(returnArray);
		}
	}


	/**
	*	Perform Gauss Elimination on an n x n+1 matrix
	**/
	public static double[] gaussElim(double[][] matrix) {
		double tempDouble;
		int n= matrix.length;
		double[] solutionVector = new double[n];

		for(int i=0; i<n; i++) {
			// Begin pivoting routine
			int maxIndex = i;
			for(int j=i; j<n; j++) {
				if(Math.abs(matrix[j][i]) > Math.abs(matrix[maxIndex][i])) {
					maxIndex = j;
				}
			}
			if(maxIndex != i) {
				double[] tempArray = matrix[i];
				matrix[i] = matrix[maxIndex];
				matrix[maxIndex] = tempArray;
			}
			// End pivoting routine
			for(int j=n; j>=i; j--) {
				matrix[i][j] = matrix[i][j]/matrix[i][i];
			}
			for(int j=(i+1); j<=(n-1);j++) {
				for(int k=n; k>=i; k--) {
					matrix[j][k] = matrix[j][k] - (matrix[j][i] * matrix[i][k]);
				}
			}
		}
		// back substitution
		for(int i= (n-1); i>=0; i--) {
			tempDouble = matrix[i][n];
			for(int j=(n-1); j>i; j--) {
				tempDouble = tempDouble - (matrix[i][j] * solutionVector[j]);
			}
			solutionVector[i] = tempDouble/matrix[i][i];
		}
		return(solutionVector);
	}


	/**
	*	Hyperbolic cosine function
	*	@return (e<SUP>x</SUP>+e<SUP>-x</SUP>)/2.0
	**/
	public static double cosh(double x) {
		return( (Math.exp(x) + Math.exp(-1*x))/2.0 );
	}

	/**
	*	Hyperbolic sine function
	*	@return (e<SUP>x</SUP>-e<SUP>-x</SUP>)/2.0
	**/
	public static double sinh(double x) {
		return( (Math.exp(x) - Math.exp(-1*x))/2.0 );
	}

	/**
	*	Inverse Hyperbolic cosine function
	**/
	public static double acosh(double x) {
		return( Math.log(x + Math.sqrt(Math.pow(x,2)-1)));
	}

	/**
	*	Inverse Hyperbolic cosine
	**/
	public static double asinh(double x) {
		return( Math.log(x + Math.sqrt(Math.pow(x,2)+1)));
	}


	/**
	*	Legendre elliptic integral of the first kind
	*	as defined at http://www.scf.fundp.ac.be/~jpvigner/homepage/sources/recipes/recipes.html
	**/

	public static double ellf(double phi, double ak) {
		double ellf, s, piontwo=Math.PI/2.0,phitmp;
		if(phi <= piontwo) {
			ellf=ellfraw(phi,ak);
		}
		else {
			phitmp=2*piontwo-phi;
			ellf=2*ellfraw(piontwo,ak)-ellfraw(phitmp,ak);
		}
		return(ellf);
	}

	private static double ellfraw(double phi, double ak) {
		double ellfraw;
		double s;
		s = Math.sin(phi);
		ellfraw = s*rf(Math.pow(Math.cos(phi),2),(1.-s*ak)*(1.+s*ak),1.);
		return(ellfraw);
	}

	/**
	*	Legendre elliptic integral of the third kind
	*	as defined at http://www.scf.fundp.ac.be/~jpvigner/homepage/sources/recipes/recipes.html
	**/
	public static double ellpi(double phi,double en,double ak) {
		double ellpi;
		double phitmp,piontwo;
		piontwo=1.570796;
		if(phi <= piontwo) {
			ellpi=ellpiraw(phi,en,ak);
		} else {
			phitmp = 2*piontwo-phi;
			ellpi = 2*ellpiraw(piontwo,en,ak)-ellpiraw(phitmp,en,ak);
		}
		return(ellpi);
	}

	private static double ellpiraw(double phi, double en, double ak) {
		double ellpiraw;
		double cc,enss,q,s;
		s=Math.sin(phi);
		enss=en*s*s;
		//System.out.println("enss " + enss);
		//System.out.println("en " + en);
		//System.out.println("s " + s);
		cc=Math.pow(Math.cos(phi),2);
		q=(1.0-s*ak)*(1.0+s*ak);
		ellpiraw=s*(rf(cc,q,1.0)-enss*rj(cc,q,1.0,1.0+enss)/3.0);
		return(ellpiraw);
	}

	/**
	*	Carlson's elliptic integral of the first kind (6.11)
	*	as defined at http://www.scf.fundp.ac.be/~jpvigner/homepage/sources/recipes/recipes.html
	**/
	public static double rf(double x,double y,double z) {
		double rf;
		final double ERRTOL=.08, TINY=1.5e-38, BIG=3.e37, THIRD=1./3.,
					 C1=1./24., C2=.1, C3=3./44., C4=1./14.;
		double alamb,ave,delx,dely,delz,e2,e3,sqrtx,sqrty,sqrtz,xt,yt,zt;
		if(Math.min(Math.min(x,y),z) < 0. || Math.min(Math.min(x+y,x+z),y+z) < TINY || Math.max(Math.min(x,y),z) > BIG) {
			// pause 'invalid arguments in rf'
			return(0.);
		}
	    xt=x;
	    yt=y;
	    zt=z;
		do {
			sqrtx=Math.sqrt(xt);
			sqrty=Math.sqrt(yt);
			sqrtz=Math.sqrt(zt);
			alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz;
			xt=.25*(xt+alamb);
			yt=.25*(yt+alamb);
			zt=.25*(zt+alamb);
			ave=THIRD*(xt+yt+zt);
			delx=(ave-xt)/ave;
			dely=(ave-yt)/ave;
			delz=(ave-zt)/ave;
		} while(Math.max(Math.max(Math.abs(delx),Math.abs(dely)),Math.abs(delz)) > ERRTOL);

	    e2=delx*dely-Math.pow(delz,2);
	    e3=delx*dely*delz;
	    rf=(1.+(C1*e2-C2-C3*e3)*e2+C4*e3)/Math.sqrt(ave);
        return(rf);
	}

	/**
	*	Carlson's elliptic integral of the third kind (6.11)
	*	as defined at http://www.scf.fundp.ac.be/~jpvigner/homepage/sources/recipes/recipes.html
	**/
	public static double rj(double x, double y, double z, double p) {
		double rj;
		final double ERRTOL=.05, TINY=2.5e-13, BIG=9.0e11, C1=3./14.,
					 C2=1./3., C3=3./22.,C4=3./26., C5=.75*C3, C6=1.5*C4, C7=.5*C2, C8=C3+C3;
		double a=0,alamb,alpha,ave,b=0,beta,delp,delx,dely,delz,ea,eb,ec,ed,ee,
     		   fac,pt,rcx=0,rho,sqrtx,sqrty,sqrtz,sum,tau,xt,yt,zt,rc,rf;
		if((Math.min(Math.min(x,y),z) < 0.)||(Math.min(Math.min(x+y,x+z),Math.min(y+z,Math.abs(p))) < TINY)||(Math.max(Math.max(x,y),Math.max(z,Math.abs(p))) > BIG)) {
			//  System.out.println("invalid arguments in rj");
		}
		sum=0.;
		fac=1.;
	    if(p > 0.) {
			xt=x;
			yt=y;
			zt=z;
			pt=p;
		} else {
			xt=Math.min(Math.min(x,y),z);
			zt=Math.max(Math.min(x,y),z);
			yt=x+y+z-xt-zt;
			a=1./(yt-p);
			b=a*(zt-yt)*(yt-xt);
			pt=yt+b;
			rho=xt*zt/yt;
			tau=p*pt/yt;
			//System.out.println("rho " + rho);
			//System.out.println("tau " + tau);
			//System.out.println("rc(rho,tau) " + rc(rho,tau));
       		rcx=rc(rho,tau);
		}
		//System.out.println("rcx mid " + rcx);
		do {
			sqrtx=Math.sqrt(xt);
			sqrty=Math.sqrt(yt);
			sqrtz=Math.sqrt(zt);
			alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz;
			alpha=Math.pow((pt*(sqrtx+sqrty+sqrtz)+sqrtx*sqrty*sqrtz),2);
			beta=pt*Math.pow((pt+alamb),2);
			sum=sum+fac*rc(alpha,beta);
			fac=.25*fac;
			xt=.25*(xt+alamb);
			yt=.25*(yt+alamb);
			zt=.25*(zt+alamb);
			pt=.25*(pt+alamb);
			ave=.2*(xt+yt+zt+pt+pt);
			delx=(ave-xt)/ave;
			dely=(ave-yt)/ave;
			delz=(ave-zt)/ave;
			delp=(ave-pt)/ave;
		} while(Math.max(Math.max(Math.abs(delx),Math.abs(dely)),Math.max(Math.abs(delz),Math.abs(delp))) > ERRTOL);
		ea=delx*(dely+delz)+dely*delz;
		eb=delx*dely*delz;
		ec=Math.pow(delp,2);
		ed=ea-3.*ec;
		ee=eb+2.*delp*(ea-ec);
		rj=3.*sum+fac*(1.+ed*(-C1+C5*ed-C6*ee)+eb*(C7+delp*(-C8+delp*C4))+delp*ea*(C2-delp*C3)-C2*delp*ec)/(ave*Math.sqrt(ave));
		//System.out.println("rj " + rj);
		//System.out.println("rf " + rf(xt,yt,zt));
		//System.out.println("a " + a);
		//System.out.println("b " + b);
		//System.out.println("rj " + rj);
		//System.out.println("rcx " + rcx);
		if (p <= 0.) rj=a*(b*rj+3.*(rcx-rf(xt,yt,zt)));
		//System.out.println("rj " + rj);
      	return(rj);
	}

	/**
	*	Carlson's degenerate elliptic integral (6.11)
	*	as defined at http://www.scf.fundp.ac.be/~jpvigner/homepage/sources/recipes/recipes.html
	**/
	public static double rc(double x,double y) {
		double rc;
		final double ERRTOL=.04,TINY=1.69e-38,SQRTNY=1.3e-19,BIG=3.E37,TNBG=TINY*BIG,
					 COMP1=2.236/SQRTNY,COMP2=TNBG*TNBG/25.,THIRD=1./3.,
     				 C1=.3,C2=1./7.,C3=.375,C4=9./22.;
     	double alamb,ave,s,w,xt,yt;
     	if((  (x < 0.)||(y==0.)||((x+Math.abs(y))<TINY)||((x+Math.abs(y)) > BIG)||((y<-COMP1)&&(x>0.)&&(x<COMP2)))) {
			//System.out.println("invalid arguments in rc");
		}
		if(y > 0.) {
			xt=x;
			yt=y;
			w=1.;
		}
		else {
			xt=x-y;
			yt=-y;
			w=Math.sqrt(x)/Math.sqrt(xt);
		}
		do {
		        alamb=2.*Math.sqrt(xt)*Math.sqrt(yt)+yt;
		        xt=.25*(xt+alamb);
		        yt=.25*(yt+alamb);
		        ave=THIRD*(xt+yt+yt);
		        s=(yt-ave)/ave;
		} while(Math.abs(s) > ERRTOL);
		rc=w*(1.+s*s*(C1+s*(C2+s*(C3+s*C4))))/Math.sqrt(ave);
		return(rc);
	}


	/**
	*	Roundoff the inputDouble to the prescribed number of decimal places
	**/
	public static double roundOff(double inputDouble, int places) {
		return(Math.round(inputDouble*(Math.pow(10,places)))/(Math.pow(10,places)));
	}

/*
    void PyramidSort(int a[]) throws Exception {
		int arrayMinLoc, arrayMaxLoc;
		int temp=0, i, j;
		for(i=0; i< a.length/2; i++) {
			arrayMinLoc = i;
			arrayMaxLoc = i;
			for(j=i+1; j< a.length-i; j++) {

				if(a[j] < a[arrayMinLoc]) {
					arrayMinLoc=j;
				} else if(a[j] > a[arrayMaxLoc]) {
					arrayMaxLoc=j;
				}
			}

			temp=a[i];
			a[i]=a[arrayMinLoc];
			a[arrayMinLoc]=temp;

			temp=a[a.length-1-i];
			a[a.length-1-i]=a[arrayMaxLoc];
			a[arrayMaxLoc]=temp;

			pause(i,j);
		}
	}
*/
/*

    public static sort2(double a[], double b[]) {
		int j;
		int limit = a.length;
		int st = -1;
		while (st < limit) {
			boolean flipped = false;
			st++;
			limit--;
			for (j = st; j < limit; j++) {
				if (a[j] > a[j + 1]) {
					int T = a[j];
					a[j] = a[j + 1];
					a[j + 1] = T;
					//move b to follow
					T = b[j];
					b[j] = b[j + 1];
					b[j + 1] = T;
					flipped = true;
				}
			}
			if (!flipped) {
				return;
			}
			for (j = limit; --j >= st;) {
				if (a[j] > a[j + 1]) {
					int T = a[j];
					a[j] = a[j + 1];
					a[j + 1] = T;
					//move b to follow
					T = b[j];
					b[j] = b[j + 1];
					b[j + 1] = T;
					flipped = true;
				}
			}
			if (!flipped) {
				return;
			}
		}
    }

*/

}


