import java.awt.*;
import java.awt.event.*;
//import java.awt.geom.*;

/**
*	A JDK1.1-compatible plotting frame
*
*	@author Rob Falck - Case Western Reserve University
*
*/

public class GraphPane extends Canvas {
	/** The maximum value on the horizontal axis in the visible graph **/
	public double xMax;
	/** The minimum value on the horizontal axis in the visible graph **/
	public double xMin;
	/** The maximum value on the vertical axis in the visible graph **/
	public double yMax;
	/** The minimum value on the vertical axis in the visible graph **/
	public double yMin;
	private boolean isNew = true;

	/**	The text label for the horizontal axis **/
	public String xLabel = new String();
	/** The text label for the vertical axis **/
	public String yLabel = new String();

	/**
	*	The width of the graph in pixels
	*	currently hard-coded due to incompatibility issues
	**/
	double paneWidth= 450;
	/**
	*	The height of the graph in pixels
	*	currently hard-coded due to incompatibility issues
	**/
	double paneHeight= 250;

	/** The index of the case that is currently being traced**/
	public int traceCase = 0;
	/** The tracing point, tracked in data coordinates **/
	protected double traceX=0, traceY=0;

	/** Initializes a new GraphPane **/
	public GraphPane() {
		this.setBackground(Color.lightGray);

		this.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {

				if(e.getKeyCode() == 16) return;
				if(Cosmo.traceCheckbox.getState()==false) return;

				// If Up is Pressed
				if(e.getKeyCode() == 38) {
					int oldTrace = traceCase;
					do {
						traceCase +=1;
						if(traceCase >= Cosmo.cases.length) {
							traceCase = 0;
						}
						if(traceCase == oldTrace) break;
					} while((Cosmo.cases[traceCase].getH0() <= 0)&&((Cosmo.cases[traceCase].getOmegaM() <= 0)||(Cosmo.cases[traceCase].getOmegaL() <= 0)));
					updateTraceY();
				}
				// If Down is Pressed
				else if(e.getKeyCode() == 40) {
					int oldTrace = traceCase;
					do {
						traceCase -=1;
						if(traceCase < 0) {
							traceCase = Cosmo.cases.length-1;
						}
						if(traceCase == oldTrace) break;
					} while((Cosmo.cases[traceCase].getH0() <= 0)&&((Cosmo.cases[traceCase].getOmegaM() <= 0)||(Cosmo.cases[traceCase].getOmegaL() <= 0)));
					updateTraceY();
				}
				// If Right is Pressed
				else if(e.getKeyCode() == 39) {
					if(Cosmo.plotType ==2) {
						do {
							if(e.isShiftDown()) Cosmo.cases[traceCase].traceIndex += 1;
							else Cosmo.cases[traceCase].traceIndex += 10;
							if(Cosmo.cases[traceCase].traceIndex >= Cosmo.cases[traceCase].tplot.length) {
								Cosmo.cases[traceCase].traceIndex = Cosmo.cases[traceCase].tplot.length-1;
								break;
							}
						} while(Cosmo.cases[traceCase].tplot[Cosmo.cases[traceCase].traceIndex] == 0);

					} else {
						if(traceX >= xMax) return;
						if(e.isShiftDown()) traceX += .01;
						else traceX += 0.1;
						if(traceX >= xMax) traceX = xMax;
					}
					updateTraceY();
				}
				// If Left is Pressed
				else if(e.getKeyCode() == 37) {
					if(Cosmo.plotType ==2) {
						int lastGood = Cosmo.cases[traceCase].traceIndex;
						do {
							if(e.isShiftDown()) Cosmo.cases[traceCase].traceIndex -= 1;
							else Cosmo.cases[traceCase].traceIndex -= 10;
							if(Cosmo.cases[traceCase].traceIndex <= 0) {
								Cosmo.cases[traceCase].traceIndex = 0;
								break;
							}
						} while(Cosmo.cases[traceCase].tplot[Cosmo.cases[traceCase].traceIndex] == 0);
					} else {
						if(traceX <= xMin) return;
						if(e.isShiftDown()) traceX -= .01;
						else traceX -= 0.1;
						if(traceX <= xMin) traceX = xMin;
					}
					updateTraceY();
				}
				repaint();
			}
		});

		addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				if(Cosmo.traceCheckbox.getState() == true) {
					Cosmo.traceCheckbox.setForeground(Color.black);
				}
			}

			public void focusLost(FocusEvent e) {
				Cosmo.traceCheckbox.setForeground(Color.gray);
			}
		});

		addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				requestFocus();
			}
			public void mousePressed(MouseEvent e) {
				for(int i=0; i< Cosmo.cases.length; i++) {
					Cosmo.cases[i].h0.setValue();
					Cosmo.cases[i].omegaM.setValue();
					Cosmo.cases[i].omegaL.setValue();
				}
				Cosmo.dataIsNew = true;
				repaint();
			}
			public void mouseReleased(MouseEvent e) {
			}
			public void mouseEntered(MouseEvent e) {
			}
			public void mouseExited(MouseEvent e) {
			}
		});
	}


	public void updateTraceY() {
		if(Cosmo.traceCheckbox.getState() == true) {
			if((Cosmo.plotType == 0)||(Cosmo.plotType==1)) {
				traceY = getOutput(Cosmo.cases[traceCase],traceX);
				Cosmo.traceLabel.setText(FMath.roundOff(traceX,3) + "," + FMath.roundOff(traceY,3));
			} else if(Cosmo.plotType == 2){
				Cosmo.traceLabel.setText(FMath.roundOff(Cosmo.cases[traceCase].tplot[Cosmo.cases[traceCase].traceIndex],3) + "," + FMath.roundOff(Cosmo.cases[traceCase].aplot[Cosmo.cases[traceCase].traceIndex],3));
			}
			repaint();
		}
	}



	/** Paints the Graphpane
	*	Any valid plots will be drawn
	**/
	public void paint(Graphics g) {
		if(Cosmo.dataIsNew == true) scaleGraph();
		g.setColor(Color.lightGray);
		g.fillRect(0,0,(int)paneWidth-1,(int)paneHeight-1);
		g.setColor(Color.gray);
		g.drawRect(0,0,(int)paneWidth-1,(int)paneHeight-1);
		g.setFont(new Font("Arial", Font.PLAIN, 10));
		// Draw the y axis and label
		if(Cosmo.plotType <2) {
			for(double i=1.0; i<yMax; i+= 1.0) {
				g.setColor(Color.gray);
				g.drawLine(0,(int)dataToScreen(0,i).y,(int)paneWidth,(int)dataToScreen(0,i).y);
				g.setColor(Color.black);
				g.drawString(FMath.roundOff(i,1)+"",3,(int)dataToScreen(0,i).y-1);
			}
			// Draw the x axis and label
			for(double i=1; i<xMax; i+= 1.0) {
				g.setColor(Color.gray);
				g.drawLine((int)dataToScreen(i,0).x,0,(int)dataToScreen(i,.05).x,(int)paneHeight);
				g.setColor(Color.black);
				g.drawString(FMath.roundOff(i,1)+"",(int)dataToScreen(i,.05).x+2,(int)paneHeight-5);
			}
		} else {
			for(double i=1.0; i<yMax; i+= 1.0) {
				g.setColor(Color.gray);
				g.drawLine(0,(int)dataToScreen(0,i).y,(int)paneWidth,(int)dataToScreen(0,i).y);
				g.setColor(Color.black);
				g.drawString(FMath.roundOff(i,0)+"",3,(int)dataToScreen(0,i).y-1);
			}
			// Draw the x axis and label

			for(double i=xMin; i<xMax; i+= 10) {
				g.setColor(Color.gray);
				if(i==0) g.setColor(new Color(20,36,75));
				g.drawLine((int)dataToScreen(i,0).x,0,(int)dataToScreen(i,0).x,(int)paneHeight);
				g.setColor(Color.black);
				g.drawString((int)i+"",(int)dataToScreen(i,0).x+2,(int)paneHeight-5);
			}
		}

		// Draw the plots
		Point newPoint = new Point(), oldPoint = new Point();

		double y=0;
		int NaNCount=0;

		for(int i=0; i< Cosmo.cases.length; i++) {
			NaNCount = 0;
			if((Cosmo.cases[i].h0.value > 0)&&((Cosmo.cases[i].omegaM.value > 0)||(Cosmo.cases[i].omegaL.value > 0))) {
				isNew = false;
				g.setColor(Cosmo.cases[i].color);
				if((Cosmo.plotType == 0)||(Cosmo.plotType == 1)) {
					for(double j=xMin; j<=xMax; j+= (xMax-xMin)/400.0) {
						y = getOutput(Cosmo.cases[i],j);
						if((y != y)||(y == Double.POSITIVE_INFINITY)) {
							NaNCount++;
						} else  {
							newPoint = dataToScreen(j,y);
							if(j>xMin) g.drawLine((int)oldPoint.x,(int)oldPoint.y,(int)newPoint.x,(int)newPoint.y);
							oldPoint.setLocation(newPoint);
						}
						if(NaNCount == 400) {
							g.setFont(new Font("Arial", Font.PLAIN, 15));
							g.drawString("NO BIG BANG",(int)(paneWidth*3.0/8.0),5+(15*(i+1)));
							break;
						}
					}
				} else if(Cosmo.plotType ==2){
					//getTplot(Cosmo.cases[i]);
					oldPoint = dataToScreen(Cosmo.cases[i].tplot[0],Cosmo.cases[i].aplot[0]);
					for(int j=1; j< Cosmo.cases[i].tplot.length; j++) {
						if(Cosmo.cases[i].tplot[j] != 0) {
							newPoint = dataToScreen(Cosmo.cases[i].tplot[j],Cosmo.cases[i].aplot[j]);
							g.drawLine((int)oldPoint.x,(int)oldPoint.y,(int)newPoint.x,(int)newPoint.y);
							oldPoint.setLocation(newPoint);
						}
					}
				}
			}
		}

		if(isNew == true) {
			g.setFont(new Font("Arial", Font.PLAIN, 12));
			g.drawString("Credits:",100,105);
			g.drawString("- Scientific Development: Chris Mihos (CWRU)",100,120);
			g.drawString("- Java Programming: Rob Falck",100,135);
			g.drawString("- Web Design: Rob Falck, Cameron McBride",100,150);
		}

		g.setColor(Color.black);
		if(Cosmo.traceCheckbox.getState()==true) {
			if((Cosmo.plotType == 0)||(Cosmo.plotType == 1)) {
				g.drawOval(dataToScreen(traceX,traceY).x-4,dataToScreen(traceX,traceY).y-4,8,8);
			} else if (Cosmo.plotType == 2) {
				g.drawOval(dataToScreen(Cosmo.cases[traceCase].tplot[Cosmo.cases[traceCase].traceIndex],Cosmo.cases[traceCase].aplot[Cosmo.cases[traceCase].traceIndex]).x-4,dataToScreen(Cosmo.cases[traceCase].tplot[Cosmo.cases[traceCase].traceIndex],Cosmo.cases[traceCase].aplot[Cosmo.cases[traceCase].traceIndex]).y-4,8,8);
			}
		}

		// Plot any data points
		g.setColor(Color.yellow);
		if(Cosmo.plotType == 1) {
			for(int i=0; i< Cosmo.dataPoints.length; i++) {
				Cosmo.dataPoints[i].xValue.setValue();
				Cosmo.dataPoints[i].yValue.setValue();
				Cosmo.dataPoints[i].uncertainty.setValue();

				Point a=new Point(), b=new Point();
				a = dataToScreen( Cosmo.dataPoints[i].getXValue() , Cosmo.dataPoints[i].getYValue() + Cosmo.dataPoints[i].getUncertainty() );
				b = dataToScreen( Cosmo.dataPoints[i].getXValue() , Cosmo.dataPoints[i].getYValue() - Cosmo.dataPoints[i].getUncertainty() );
				g.drawLine(a.x,a.y,b.x,b.y);
				g.drawLine(a.x-2,a.y,a.x+2,a.y);
				g.drawLine(b.x-2,b.y,b.x+2,b.y);
				a = dataToScreen( Cosmo.dataPoints[i].getXValue() , Cosmo.dataPoints[i].getYValue());
				g.drawOval(a.x-1,a.y-1,2,2);
			}
		}
	}

	/** Given the x,y coordinates of a data point, converts
	*	them too an x,y point in the graphing pane's coordinate
	*	system.
	**/
	public Point dataToScreen(double x, double y) {
		double screenX = 0,screenY = 0;
		// paneWidth = this.getSize().getWidth();
		//paneHeight = this.getSize().getHeight();
		double deltaX = xMax - xMin;
		double deltaY = yMax - yMin;
		screenX = (paneWidth/deltaX)*(x - xMin);
		screenY = (paneHeight) - ( (paneHeight/deltaY)*(y - yMin));
		return(new Point((int)screenX,(int)screenY));
	}

	/**	Given the x,y coorinates of a point on the graphing
	*	pane's coordinate system, converts them to an x,y
	*	point in the data's coordinate system
	**/
	public double[] screenToData(Point inputPoint) {
		double[] output = new double[2];
		double dataX =0, dataY =0;
		// paneWidth = this.getSize().getWidth();
		//  paneHeight = this.getSize().getHeight();
		double deltaX = xMax - xMin;
		double deltaY = yMax - yMin;
		dataX = inputPoint.x*(deltaX/paneWidth)+xMin;
		dataY = ((-inputPoint.y*deltaY)+(paneHeight*deltaY)+(paneHeight*yMin))/paneHeight;
		output[0] = dataX;
		output[1] = dataY;
		return(output);
	}




	public double getOutput(Case c, double z) {
		double y=0;
		if((c.h0.value <= 0)&&((c.omegaM.value <= 0)||(c.omegaL.value <= 0))) {
			return(0);
		}
		if(Cosmo.plotType == 0) {
			y = Cosmo.z2t(c,0)-Cosmo.z2t(c,z);
		} else {
			y = Cosmo.z2t(c,z);
		}
		y = y * (100./c.getH0()) * 10.;
		return(y);
	}


	public void getTplot(Case c) {
		for(int i=0; i<c.tplot.length; i++) {
			c.tplot[i] =0;
			c.aplot[i] =0;
		}
		//double[] Cosmo.cases[i].tplot = new double[1000], Cosmo.cases[i].aplot=new double[1000];
		double h0 = c.getH0();
		double h= h0/100.;
		double t0 = 10./h;
		double omegam = c.getOmegaM();
		double omegal = c.getOmegaL();
		int i;
		double a,t,dt,dadt;
		double twopi=2*3.1415927;
		if((omegal == 0)&&(omegam > 1)) {
			double tnow=0.5*(omegam/Math.pow((omegam-1),1.5))*(Math.acos(2/omegam-1)-2*Math.sqrt(omegam-1)/omegam);
			i=1;
			for(double x=0;x<twopi;x+=0.01) {
				t=0.5*(omegam/Math.pow((omegam-1),1.5))*(x-Math.sin(x));
				c.tplot[i]=t0*(t-tnow);
				a=0.5*(omegam/(omegam-1))*(1-Math.cos(x));
				c.aplot[i]=a;
				i=i+1;
			}
		} else {
			a=1;
			t=0;
			dt=0.01;
			i=0;
			while(a > 0.01) {
				dadt = Math.sqrt(1+omegam*(1/a-1)+omegal*(a*a-1));
				t=t-dt;
				a=a-dadt*dt;
				c.tplot[i] = t0*t;
				c.aplot[i] = a;
				i=i+1;
			}
			a=1;
			t=0;
			dt=0.01;
			while((a<4)&&(a>0)) {
				dadt = Math.sqrt(1+omegam*(1/a-1)+omegal*(a*a-1));
				t=t+dt;
				a=a+dadt*dt;
				c.tplot[i]=t0*t;
				c.aplot[i]=a;
				i=i+1;
			}
			// Sort t, rearrange a to match
			int j;
			int limit = c.tplot.length;
			int st = -1;
			while (st < limit) {
				boolean flipped = false;
				st++;
				limit--;
				for (j = st; j < limit; j++) {
					if (c.tplot[j] > c.tplot[j + 1]) {
						double T = c.tplot[j];
						c.tplot[j] = c.tplot[j + 1];
						c.tplot[j + 1] = T;
						//move b to follow
						T = c.aplot[j];
						c.aplot[j] = c.aplot[j + 1];
						c.aplot[j + 1] = T;
						flipped = true;
					}
				}
				if (!flipped) {
					return;
				}
				for (j = limit; --j >= st;) {
					if (c.tplot[j] > c.tplot[j + 1]) {
						double T = c.tplot[j];
						c.tplot[j] = c.tplot[j + 1];
						c.tplot[j + 1] = T;
						//move b to follow
						T = c.aplot[j];
						c.aplot[j] = c.aplot[j + 1];
						c.aplot[j + 1] = T;
						flipped = true;
					}
				}
				if (!flipped) {
					return;
				}
				// end sort
				for(j=0; j< c.tplot.length; j++) {
					if(c.tplot[j] != 0) {
						c.traceIndex = j;
						break;
					}
				}


			}
		}
	}



	/**	Set the horizontal axis label to s **/
	public void setXLabel(String s) {
		xLabel = s;
	}

	/**	Set the vertical axis label to s **/
	public void setYLabel(String s) {
		yLabel = s;
	}

	/**	Set the minimum value on the horizontal axis **/
	public void setXMin(double xMin) {
		this.xMin = xMin;
	}

	/**	Set the maximum value on the horizontal axis **/
	public void setXMax(double xMax) {
		this.xMax = xMax;
	}

	/**	Set the minimum value on the vertical axis **/
	public void setYMin(double yMin) {
		this.yMin = yMin;
	}

	/**	Set the maximum value on the vertical axis **/
	public void setYMax(double yMax) {
		this.yMax = yMax;
	}

	/**	Automatically set the graph's bounds to the appropriate values
	*	by rounding up to the next billion years
	**/
	public void scaleGraph() {
		double y=0;
		setYMin(0);
		setXMin(0);
		setYMax(0);
		setXMax(0);
		for(int i=0; i< Cosmo.cases.length; i++) {
			if((Cosmo.cases[i].getH0() > 0)&&((Cosmo.cases[i].getOmegaM() > 0)||(Cosmo.cases[i].getOmegaL() > 0))) {
				if(Cosmo.plotType == 2) {
					getTplot(Cosmo.cases[i]);
					setXMin(Math.min(xMin,FMath.roundOff(Cosmo.cases[i].tplot[FMath.arrayMin(Cosmo.cases[i].tplot)]-6,-1)));
					setYMin(0);
					setXMax(Math.max(xMax,FMath.roundOff(Cosmo.cases[i].tplot[FMath.arrayMax(Cosmo.cases[i].tplot)]+6,-1)));
					//setYMax(Math.max(yMax,FMath.roundOff(Cosmo.cases[i].aplot[FMath.arrayMax(Cosmo.cases[i].aplot)]+.6,0)));
					setYMax(4);
					setXMax(Math.min(xMax,100));
				} else {

					setXMax(6);
					if(Cosmo.plotType == 0) {
						y = getOutput(Cosmo.cases[i],6);
					} else if(Cosmo.plotType == 1) {
						y = getOutput(Cosmo.cases[i],0);
					}
					if((y != Double.POSITIVE_INFINITY)&&(y == y)) {
						setYMax(Math.max(this.yMax,y));
					}
				}
			}
		}
		this.yMax = Math.ceil(this.yMax);
		this.yMax = Math.min(this.yMax,100);
		if((this.yMax == 0)||(this.yMax == Double.POSITIVE_INFINITY)||(this.yMax != this.yMax)) this.setYMax(10);
		Cosmo.dataIsNew = false;
	}
}
